namespace CADOExceptions
{
	using System;
	using System.Data;
	using System.Data.SqlClient;

	/// <summary>
	///		Summary description for Class1.
	/// </summary>
	class Class1
	{
		static void Main(string[] args)
		{

			//code to load a dataset with three tables from the northwinds database
			//note this could be done with one call to a stored procedure that returns
			//multiple recordsets
			SqlConnection cnn = new SqlConnection("server=localhost;initial catalog=northwind;uid=sa;pwd=;");
			SqlCommand cmd1 = new SqlCommand("select * from orders", cnn);
			SqlCommand cmd2 = new SqlCommand("select * from customers", cnn);
			SqlCommand cmd3 = new SqlCommand("select * from [order details]",cnn);

			SqlDataAdapter da = new SqlDataAdapter(cmd1);
			DataSet ds = new DataSet();

			//fill the first table
			da.Fill(ds, "orders");

			//change the command and fill the second table
			da.SelectCommand = cmd2;
			da.Fill(ds, "customers");

			//change the command object again and fill the third table
			da.SelectCommand = cmd3;
			da.Fill(ds,"orderdetails");

			Class1 c = new Class1();


			//////////////////////////////////////////////
			////each of the following method calls demonstrates a different
			///exception.  Various examples are included showing the use of the 
			///properties and methods of the exception objects
			//////////////////////////////////////////////
/*		*/	
			//constraint exception
			c.Constraint(ds);
			c.Pause();

			//Duplicate Name exception
			c.Duplicate(ds);
			c.Pause();

			//EvaluateException
			c.Eval(ds);
			c.Pause();

			//InRowChangingEvent
			c.RowChange(ds);
			c.Pause();

			//InvalidConstraint
			c.InvalidConstr(ds);
			c.Pause();

			//InvalidExpression
			c.InvalidExpress(ds);
			c.Pause();

			//Missing Primary Key
			c.MissingKey(ds);
			c.Pause();

			//Null value not allowed
			c.NoNull(ds);
			c.Pause();
				
			//Read-only 
			//c.ReadOnly(ds);
			//c.Pause();
				
			//Row not in table
			c.RowNotInTable(ds);
			c.Pause();

			//syntax error
			c.Syntax(ds);
			c.Pause();

			//Version not available
			c.VersionNotAvailable(ds);
			c.Pause();

			//DataException
			c.DataExcept(ds);
			c.Pause();

			//sql exception, error collection and errors
			c.SqlError(cnn);
			c.Pause();

			//DeletedRow exception
			c.DelRow(ds);
			c.Pause();
/**/
			cnn.Open();
			SqlCommand cmdDel = new SqlCommand("delete customers where customerid='PARIS'",cnn);
			try
			{
				cmdDel.ExecuteNonQuery();
			}
			catch(SqlException se)
			{
				Console.WriteLine(se.ToString());
			}
			Console.ReadLine();
			cnn.Close();
			//db concurrency
			c.Concurrency(ds, da);
			c.Pause();

		}


		public void SqlError(SqlConnection cnn)
		{
			Console.WriteLine("\t\t\tSqlExeption:\n\n");

			SqlCommand errorcommand = new SqlCommand("NonexistentProc",cnn);
			errorcommand.CommandType = CommandType.StoredProcedure;
			errorcommand.Parameters.Add(new SqlParameter("@inval",SqlDbType.Int));
			errorcommand.Parameters["@inval"].Value = 5;
			
			try
			{
				cnn.Open();
				errorcommand.ExecuteNonQuery();
			}
			catch(SqlException sqe)
			{
				Console.WriteLine("Error processing stored procedure:\n");
				foreach(System.Data.SqlClient.SqlError se in sqe.Errors)
				{
					Console.WriteLine(se.Message);
					Console.WriteLine("Server:\t" + se.Server.ToString());
					Console.WriteLine("Error number:\t" + se.Number.ToString());
					Console.WriteLine("Severity:\t" + se.Class.ToString());
					Console.WriteLine("Stored proc:\t" + se.Procedure.ToString());
					Console.WriteLine("T-SQL Line number:\t" + se.LineNumber.ToString());
					Console.WriteLine("Source:\t" + se.Source.ToString());
					Console.WriteLine("State:\t" + se.State.ToString());

				}
			}

		}


		public void Concurrency(DataSet ds, SqlDataAdapter sda)
		{
			Console.WriteLine("\t\t\tDbConcurrency:\n\n");
			try
			{
				ds.Tables["customers"].PrimaryKey = new DataColumn[]{ds.Tables["customers"].Columns["customerid"]};
				ds.Tables["orders"].PrimaryKey = new DataColumn[]{ds.Tables["orders"].Columns["orderid"]};
				ds.Tables["orderdetails"].PrimaryKey = new DataColumn[]{ds.Tables["orderdetails"].Columns["orderdetailid"]};

				DataRow dr = ds.Tables["customers"].Rows.Find("PARIS");
				dr.BeginEdit();
				dr["contactname"]="Wrox Press";
				dr.EndEdit();

				sda.UpdateCommand = new SqlCommand("update customers set contactname = 'Wrox Press' where customerid = 'PARIS'",sda.SelectCommand.Connection);

				sda.Update(ds, "customers");
			}
			catch(DBConcurrencyException  dce)
			{
				PrintError(dce);
				
			}

		}

		public void DataExcept(DataSet ds)
		{
			Console.WriteLine("\t\t\tDataException:\n\n");

			try
			{
				DataRelation dr = new DataRelation("InvalidRelation", ds.Tables["customers"].Columns["customerid"], ds.Tables["customers"].Columns["customerid"]);
				
			}
			catch(DataException de)
			{
				PrintError(de);
			}
		}


		public void VersionNotAvailable(DataSet ds)
		{
			Console.WriteLine("\t\t\tVersion Not Available:\n\n");

			try
			{
				string result = ds.Tables["orders"].Rows[0]["orderid", DataRowVersion.Proposed].ToString();
			}
			catch(VersionNotFoundException vnfe)
			{
				PrintError(vnfe);
			}
		}

		public void Syntax(DataSet ds)
		{
			Console.WriteLine("\t\t\tSyntax Error:\n\n");

			try
			{
				ds.Tables["orderdetails"].Columns.Add(new DataColumn("TotalPrice",Type.GetType("System.Int32")));
				ds.Tables["orderdetails"].Columns["TotalPrice"].Expression = "unitprice */ quantity";
				//Console.WriteLine(ds.Tables["orderdetails"].Rows[0]["TotalPrice"].ToString());
			}
			catch(SyntaxErrorException se)
			{
				PrintError(se);
			}
		}


		public void RowNotInTable(DataSet ds)
		{
			Console.WriteLine("\t\t\tRow not in table:\n\n");

			DataRow dr = ds.Tables["customers"].NewRow();

			try
			{
				dr.BeginEdit();
				dr["customerid"]="WROX";
				dr.EndEdit();
				dr.AcceptChanges();
			}
			catch(RowNotInTableException rnite)
			{
				PrintError(rnite);

			}
			
		}

		public void ReadOnly(DataSet ds)
		{
			Console.WriteLine("\t\t\tRead only:\n\n");

			ds.Tables["customers"].Columns["customerid"].ReadOnly = true;

			try
			{
				ds.Tables["customers"].Rows[0].BeginEdit();
				ds.Tables["customers"].Rows[0]["customerid"] = "NewCust";
				//ds.Tables["customers"].Rows[0].EndEdit();
			}
			catch(ReadOnlyException roe)
			{
				PrintError(roe);
			}

		}
		public void NoNull(DataSet ds)
		{
			Console.WriteLine("\t\t\tNo null values:\n\n");

			try
			{
				ds.Tables["customers"].Columns["customerid"].AllowDBNull = false;
				ds.Tables["customers"].Rows.Add(new object[11]);
			}
			catch(NoNullAllowedException nnae)
			{
				PrintError(nnae);
			}

		}


		public void MissingKey(DataSet ds)
		{
			Console.WriteLine("\t\t\tMissing key:\n\n");

			//try to search for a value in the data table
			try
			{

				ds.Tables["orderdetails"].Rows.Contains(5);
			}
			catch(MissingPrimaryKeyException mpke)
			{
				PrintError(mpke);
			}
		}

		public void InvalidExpress(DataSet ds)
		{
			Console.WriteLine("\t\t\tInvalid Expression:\n\n");

			DataColumn dc = new DataColumn("totalprice",Type.GetType("System.Int32"));
			dc.Expression = "UnitPrice * Quantity";

			try
			{
				ds.Tables["customers"].Columns.Add(dc);
			}
			catch(InvalidExpressionException iee)
			{
				PrintError(iee);
			}


		}


		public void InvalidConstr(DataSet ds)
		{
			Console.WriteLine("\t\t\tInvalid Constraint:\n\n");

			try
			{
				DataRelation dr = new DataRelation("CustomerRelation",ds.Tables["customers"].Columns["customerid"], ds.Tables["orders"].Columns["orderid"]);
			}
			catch(InvalidConstraintException ice)
			{
				PrintError(ice);
			}

		}


		public void RowChange(DataSet ds)
		{
			Console.WriteLine("\t\t\tIn row changing event:\n\n");


			ds.Tables["customers"].RowChanging+= new DataRowChangeEventHandler(this.RowChange_Handler);
			ds.Tables["customers"].Rows[0]["contactname"] = "Wrox Press";

		}
		//private event handler for the the row changing event--used in conjuntion with the 
		//above method to show the errors when working in this event handler
		private void RowChange_Handler(Object sender,DataRowChangeEventArgs rcea)
		{
			try{
				Console.WriteLine("Row Changing . . . ");
				rcea.Row.BeginEdit();
			}
			catch (InRowChangingEventException irce)
			{
				PrintError(irce);

			}
		}


		public void Eval(DataSet ds)
		{
			Console.WriteLine("\t\t\tEvaluation exception:\n\n");

			DataColumn dc = new DataColumn("TotalPrice", Type.GetType("System.Int32"), "UnitPrice * Quantity");
			try
			{
				//will throw an error because there is no UnitPrice column in the orders table
				ds.Tables["orders"].Columns.Add(dc);
				Console.WriteLine(ds.Tables["orders"].Rows[0]["TotalPrice"].ToString());
			}
			catch(EvaluateException ee)
			{
				PrintError(ee);
			}

		}
		public  void Duplicate(DataSet ds)
		{
			Console.WriteLine("\t\t\tDuplicate Name:\n\n");

			try
			{
				ds.Tables["orders"].TableName="customers";
			}
			catch(DuplicateNameException dne)
			{
				PrintError(dne);
			}

		}

		public  void DelRow(DataSet ds)
		{
			Console.WriteLine("\t\t\tDeleted row:\n\n");

			ds.Tables["customers"].Rows[0].Delete();
			try
			{
				ds.Tables["customers"].Rows[0].BeginEdit();
			}
			catch(DeletedRowInaccessibleException drie)
			{
				PrintError(drie);
			}
		}


/*		public  void Foreign(DataSet ds)
		{
			Console.WriteLine("\t\t\tData Exception:\n\n");

			ForeignKeyConstraint fk = new ForeignKeyConstraint(ds.Tables["customers"].Columns["customerid"], ds.Tables["orders"].Columns["customerid"]);
			ds.Tables["orders"].Constraints.Add(fk);

			try
			{
				ds.Tables["customers"].PrimaryKey = new DataColumn[]{ds.Tables["customers"].Columns["customerid"]};

			}
			catch(DataException de)
			{
				PrintError(de);
			}
		}
*/
		public  void Constraint(DataSet ds)
		{
			Console.WriteLine("\t\t\tConstraint Exception:\n\n");

			//create a unique constraint on the orderid column and add to the constraints collection
			UniqueConstraint uc = new UniqueConstraint(ds.Tables["orders"].Columns["orderid"]);
			ds.Tables["orders"].Constraints.Add(uc);

			try
			{
				ds.Tables["orders"].Rows[1].BeginEdit();
				ds.Tables["orders"].Rows[1]["orderid"] = 10248;
				ds.Tables["orders"].Rows[1].EndEdit();
				for(int i =0;i<2;i++)
				{
					Console.WriteLine(ds.Tables["orders"].Rows[i]["orderid"].ToString());
				}

			}
			catch(ConstraintException ce)
			{
				PrintError(ce);

			}
			

		}

		//helper function to take an exception and write out its information to the screen
		private void PrintError(Exception e)
		{
			Console.WriteLine("Error Source:\n\t" + e.Source.ToString());
			Console.WriteLine("\nError Message:\n\t" + e.Message);
			Console.WriteLine("\nTarget Site:\n\t" + e.TargetSite.ToString());
			Console.WriteLine("\nStack Trace:\n" + e.StackTrace.ToString());


		}
		//helper method to pause the screen from scrolling
		private void Pause()
		{
			Console.WriteLine("Hit Enter to continue.");
			Console.ReadLine();
			Console.WriteLine("\n\n\n");
		}
	}
}
