using System;
using System.IO;
using System.Web;
using System.Data;
using System.Data.SqlClient;
using System.Configuration;

namespace PowerOfDotNet.Components
{
	/// <summary>
	/// ShopDB   Դϴ.
	/// </summary>
	public class ShopDB
	{
		public ShopDB()
		{
		}

		public SqlDataReader GetCategoryTopList()
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("Shop_GetCategoryTopList", myConnection);

			myCommand.CommandType = CommandType.StoredProcedure;

			myConnection.Open();
			SqlDataReader dr = myCommand.ExecuteReader(CommandBehavior.CloseConnection);

			return dr;
		}

		public SqlDataReader GetCategoryIndexList(int topId)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("Shop_GetCategoryIndexList", myConnection);
			myCommand.Parameters.Add(new SqlParameter("@TopID", SqlDbType.Int)).Value = topId;

			myCommand.CommandType = CommandType.StoredProcedure;

			myConnection.Open();
			SqlDataReader dr = myCommand.ExecuteReader(CommandBehavior.CloseConnection);

			return dr;
		}

		public DataSet GetReviewList(int prodId)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlDataAdapter da = new SqlDataAdapter("Shop_GetReviewList", myConnection);

			da.SelectCommand.CommandType = CommandType.StoredProcedure;
			da.SelectCommand.Parameters.Add(new SqlParameter("@ProductID", SqlDbType.Int)).Value = prodId;

			DataSet ds = new DataSet();
			da.Fill(ds, "ReviewList");

			return ds;
		}

		/*
		@ProductID int,
		@UserID char(12),
		@UserName nvarchar(20),
		@Comments nvarchar(3850)
		*/
		public int AddReview(int productId, string userId, string userName, string comment)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("Shop_AddReview", myConnection);

			myCommand.CommandType = CommandType.StoredProcedure;
			myCommand.Parameters.Add(new SqlParameter("@ProductID", SqlDbType.Int)).Value = productId;
			myCommand.Parameters.Add(new SqlParameter("@UserID", SqlDbType.Char, 12)).Value = userId;
			myCommand.Parameters.Add(new SqlParameter("@UserName", SqlDbType.NVarChar, 20)).Value = userName;
			myCommand.Parameters.Add(new SqlParameter("@Comments", SqlDbType.NVarChar, 3850)).Value = comment;

			myConnection.Open();

			int result;
			try
			{
				myCommand.ExecuteNonQuery();

				result = 1;
			}
			catch(Exception e) 
			{
				string s = e.Message;
				result = -1;
			}
			finally
			{
				myConnection.Close();
			}

			return result;
		}

		public SqlDataReader GetProduct(int prodId)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("Shop_GetProduct", myConnection);
			myCommand.Parameters.Add(new SqlParameter("@ID", SqlDbType.Int)).Value = prodId;

			myCommand.CommandType = CommandType.StoredProcedure;

			myConnection.Open();
			SqlDataReader dr = myCommand.ExecuteReader(CommandBehavior.CloseConnection);

			return dr;
		}

		public SqlDataReader GetProductOptionDefList(int optionId)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("Shop_GetProductOptionDefList", myConnection);
			myCommand.Parameters.Add(new SqlParameter("@OptionID", SqlDbType.Int)).Value = optionId;

			myCommand.CommandType = CommandType.StoredProcedure;

			myConnection.Open();
			SqlDataReader dr = myCommand.ExecuteReader(CommandBehavior.CloseConnection);

			return dr;
		}

		public SqlDataReader GetProductOptList(int prodId)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("Shop_GetProductOptList", myConnection);
			myCommand.Parameters.Add(new SqlParameter("@ProdID", SqlDbType.Int)).Value = prodId;

			myCommand.CommandType = CommandType.StoredProcedure;

			myConnection.Open();
			SqlDataReader dr = myCommand.ExecuteReader(CommandBehavior.CloseConnection);

			return dr;
		}

		public int AddProductOptDef(int optionId, string text)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("Shop_AddProductOptDef", myConnection);

			myCommand.CommandType = CommandType.StoredProcedure;
			myCommand.Parameters.Add(new SqlParameter("@OptionID", SqlDbType.Int)).Value = optionId;
			myCommand.Parameters.Add(new SqlParameter("@Text", SqlDbType.NVarChar, 150)).Value = text;

			myConnection.Open();

			int result;
			try
			{
				myCommand.ExecuteNonQuery();

				result = 1;
			}
			catch(Exception e) 
			{
				string s = e.Message;
				result = -1;
			}
			finally
			{
				myConnection.Close();
			}

			return result;
		}

		public int UpdateProductOptDef(int id, string text)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("Shop_UpdateProductOptDef", myConnection);

			myCommand.CommandType = CommandType.StoredProcedure;
			myCommand.Parameters.Add(new SqlParameter("@ID", SqlDbType.Int)).Value = id;
			myCommand.Parameters.Add(new SqlParameter("@Text", SqlDbType.NVarChar, 150)).Value = text;

			myConnection.Open();

			int result;
			try
			{
				myCommand.ExecuteNonQuery();

				result = 1;
			}
			catch(Exception e) 
			{
				string s = e.Message;
				result = -1;
			}
			finally
			{
				myConnection.Close();
			}

			return result;
		}

		public int UpdateProductOptDefOrder(int id, int sortOrder)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("Shop_UpdateProductOptDefOrder", myConnection);

			myCommand.CommandType = CommandType.StoredProcedure;
			myCommand.Parameters.Add(new SqlParameter("@ID", SqlDbType.Int)).Value = id;
			myCommand.Parameters.Add(new SqlParameter("@SortOrder", SqlDbType.Int)).Value = sortOrder;

			myConnection.Open();

			int result;
			try
			{
				myCommand.ExecuteNonQuery();
				result = 1;
			}
			catch(Exception e) 
			{
				string s = e.Message;
				result = -1;
			}
			finally
			{
				myConnection.Close();
			}

			return result;
		}

		public int DeleteProductOptDef(int id)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("Shop_DeleteProductOptDef", myConnection);

			myCommand.CommandType = CommandType.StoredProcedure;
			myCommand.Parameters.Add(new SqlParameter("@ID", SqlDbType.Int)).Value = id;

			myConnection.Open();

			int result;
			try
			{
				myCommand.ExecuteNonQuery();

				result = 1;
			}
			catch(Exception e) 
			{
				string s = e.Message;
				result = -1;
			}
			finally
			{
				myConnection.Close();
			}

			return result;
		}


		public SqlDataReader GetRcommendationList(int count)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("Shop_GetRecommendationList", myConnection);

			myCommand.CommandType = CommandType.StoredProcedure;
			myCommand.Parameters.Add(new SqlParameter("@COUNT", SqlDbType.Int)).Value = count;

			myConnection.Open();
			SqlDataReader dr = myCommand.ExecuteReader(CommandBehavior.CloseConnection);

			return dr;
		}

		public SqlDataReader GetProductOptionList()
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("Shop_GetProductOptionList", myConnection);

			myCommand.CommandType = CommandType.StoredProcedure;

			myConnection.Open();
			SqlDataReader dr = myCommand.ExecuteReader(CommandBehavior.CloseConnection);

			return dr;
		}

		public SqlDataReader GetBestProdectList()
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("Shop_GetBestProdectList", myConnection);

			myCommand.CommandType = CommandType.StoredProcedure;

			myConnection.Open();
			SqlDataReader dr = myCommand.ExecuteReader(CommandBehavior.CloseConnection);

			return dr;
		}

		public SqlDataReader GetBrandList()
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("Shop_GetBrandList", myConnection);

			myCommand.CommandType = CommandType.StoredProcedure;

			myConnection.Open();
			SqlDataReader dr = myCommand.ExecuteReader(CommandBehavior.CloseConnection);

			return dr;
		}

		/*
		public SqlDataReader GetProductListByIndexId(int indexId)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("Shop_GetProductListByIndexId", myConnection);
			myCommand.Parameters.Add(new SqlParameter("@IndexID", SqlDbType.Int)).Value = indexId;

			myCommand.CommandType = CommandType.StoredProcedure;

			myConnection.Open();
			SqlDataReader dr = myCommand.ExecuteReader(CommandBehavior.CloseConnection);

			return dr;
		}
		*/
		public DataSet GetProductListByIndexId(int indexId)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlDataAdapter da = new SqlDataAdapter("Shop_GetProductListByIndexId", myConnection);

			da.SelectCommand.CommandType = CommandType.StoredProcedure;
			da.SelectCommand.Parameters.Add(new SqlParameter("@IndexID", SqlDbType.Int)).Value = indexId;

			DataSet ds = new DataSet();
			da.Fill(ds, "ProductList");

			return ds;
		}

		public int DeleteIProd(int productId)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("Shop_DeleteProduct", myConnection);
			myCommand.Parameters.Add(new SqlParameter("@ProductID", SqlDbType.Int)).Value = productId;

			myCommand.CommandType = CommandType.StoredProcedure;

			myConnection.Open();

			SqlTransaction stTrans = myConnection.BeginTransaction();
			//Ʈ
			myCommand.Transaction = stTrans;

			int result;
			try
			{
				myCommand.ExecuteNonQuery();

				string strProdNum = "0000" + productId;
				strProdNum = strProdNum.Substring(strProdNum.Length-5);
				string imgPathFront = HttpContext.Current.Server.MapPath("/Shop/Images/Products/") + "ProdImg";
				string imagePath;

				imagePath = imgPathFront + "S_" + strProdNum + ".jpg";
				if (File.Exists(imagePath))
					File.Delete(imagePath);

				imagePath = imgPathFront + "M_" + strProdNum + ".jpg";
				if (File.Exists(imagePath))
					File.Delete(imagePath);

				imagePath = imgPathFront + "L_" + strProdNum + ".jpg";
				if (File.Exists(imagePath))
					File.Delete(imagePath);

				stTrans.Commit();
				result = 1;
			}
			catch(Exception e) 
			{
				string s = e.Message;
				stTrans.Rollback();
				result = -1;
			}
			finally
			{
				myConnection.Close();
			}

			return result;
		}

		public int UpdateProduct(int prodId, string name, string description, int price, 
			int discountPercent, bool absence, HttpPostedFile imageS, 
			HttpPostedFile imageM, HttpPostedFile imageL,
			int brandId, int weight, string stone)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand addProdCommand = new SqlCommand("Shop_UpdateProduct", myConnection);

			addProdCommand.CommandType = CommandType.StoredProcedure;
			addProdCommand.Parameters.Add(new SqlParameter("@ID", SqlDbType.Int)).Value = prodId;
			addProdCommand.Parameters.Add(new SqlParameter("@Name", SqlDbType.NVarChar, 40)).Value = name;
			addProdCommand.Parameters.Add(new SqlParameter("@Description", SqlDbType.NText)).Value = description;
			addProdCommand.Parameters.Add(new SqlParameter("@Price", SqlDbType.Money)).Value = price;
			addProdCommand.Parameters.Add(new SqlParameter("@DiscountPercent", SqlDbType.Int)).Value = discountPercent;
			addProdCommand.Parameters.Add(new SqlParameter("@Absence", SqlDbType.Bit)).Value = absence;
			addProdCommand.Parameters.Add(new SqlParameter("@BrandID", SqlDbType.Int)).Value = brandId;
			addProdCommand.Parameters.Add(new SqlParameter("@Weight", SqlDbType.Int)).Value = weight;
			addProdCommand.Parameters.Add(new SqlParameter("@Stone", SqlDbType.NVarChar, 50)).Value = stone;

			myConnection.Open();

			SqlTransaction stTrans = myConnection.BeginTransaction();
			//Ʈ
			addProdCommand.Transaction = stTrans;

			int result;
			try
			{
				addProdCommand.ExecuteNonQuery();

				string strProdNum = "0000" + prodId;
				strProdNum = strProdNum.Substring(strProdNum.Length-5);
				string imgPathFront = HttpContext.Current.Server.MapPath("/Shop/Images/Products/") + "ProdImg";
				if (imageS.FileName != String.Empty)
				{
					//jpgϸ Ѵ.
					if (imageS.FileName.Substring(imageS.FileName.Length-3).ToLower() != "jpg")
						throw new ArgumentException();

					File.Delete(imgPathFront + "S_" + strProdNum + ".jpg");
					imageS.SaveAs(imgPathFront + "S_" + strProdNum + ".jpg");
				}

				if (imageM.FileName != String.Empty)
				{
					//jpgϸ Ѵ.
					if (imageS.FileName.Substring(imageS.FileName.Length-3).ToLower() != "jpg")
						throw new ArgumentException();

					File.Delete(imgPathFront + "M_" + strProdNum + ".jpg");
					imageM.SaveAs(imgPathFront + "M_" + strProdNum + ".jpg");
				}

				if (imageL.FileName != String.Empty)
				{
					//jpgϸ Ѵ.
					if (imageS.FileName.Substring(imageS.FileName.Length-3).ToLower() != "jpg")
						throw new ArgumentException();

					File.Delete(imgPathFront + "D_" + strProdNum + ".jpg");
					imageL.SaveAs(imgPathFront + "L_" + strProdNum + ".jpg");
				}

				stTrans.Commit();
				result = 1;
			}
			catch(ArgumentException ae) 
			{
				string s = ae.Message;
				stTrans.Rollback();
				result = -2;
			}
			catch(Exception e) 
			{
				string s = e.Message;
				stTrans.Rollback();
				result = -1;
			}
			finally
			{
				myConnection.Close();
			}
			return result;
		}

		public int AddProduct(int indexId, string name, string description, int price, 
			int discountPercent, bool absence, HttpPostedFile imageS, 
			HttpPostedFile imageM, HttpPostedFile imageL, 
			int brandId, int weight, string stone)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand addProdCommand = new SqlCommand("Shop_AddProduct", myConnection);

			addProdCommand.CommandType = CommandType.StoredProcedure;
			addProdCommand.Parameters.Add("RETURN_VALUE", SqlDbType.Int).Direction = ParameterDirection.ReturnValue;
			addProdCommand.Parameters.Add(new SqlParameter("@IndexID", SqlDbType.Int)).Value = indexId;
			addProdCommand.Parameters.Add(new SqlParameter("@Name", SqlDbType.NVarChar, 40)).Value = name;
			addProdCommand.Parameters.Add(new SqlParameter("@Description", SqlDbType.NText)).Value = description;
			addProdCommand.Parameters.Add(new SqlParameter("@Price", SqlDbType.Money)).Value = price;
			addProdCommand.Parameters.Add(new SqlParameter("@DiscountPercent", SqlDbType.Int)).Value = discountPercent;
			addProdCommand.Parameters.Add(new SqlParameter("@Absence", SqlDbType.Bit)).Value = absence;
			addProdCommand.Parameters.Add(new SqlParameter("@BrandID", SqlDbType.Int)).Value = brandId;
			addProdCommand.Parameters.Add(new SqlParameter("@Weight", SqlDbType.Int)).Value = weight;
			addProdCommand.Parameters.Add(new SqlParameter("@Stone", SqlDbType.NVarChar, 50)).Value = stone;

			myConnection.Open();

			SqlTransaction stTrans = myConnection.BeginTransaction();
			//Ʈ
			addProdCommand.Transaction = stTrans;

			int result;
			try
			{
				addProdCommand.ExecuteNonQuery();
				int newId = Convert.ToInt32(addProdCommand.Parameters["RETURN_VALUE"].Value);

				string strProdNum = "0000" + newId;
				strProdNum = strProdNum.Substring(strProdNum.Length-5);
				string imgPathFront = HttpContext.Current.Server.MapPath("/Shop/Images/Products/") + "ProdImg";
				if (imageS.FileName != String.Empty)
				{
					//jpgϸ Ѵ.
					if (imageS.FileName.Substring(imageS.FileName.Length-3).ToLower() != "jpg")
						throw new ArgumentException();

					imageS.SaveAs(imgPathFront + "S_" + strProdNum + ".jpg");
				}

				if (imageM.FileName != String.Empty)
				{
					//jpgϸ Ѵ.
					if (imageS.FileName.Substring(imageS.FileName.Length-3).ToLower() != "jpg")
						throw new ArgumentException();

					imageM.SaveAs(imgPathFront + "M_" + strProdNum + ".jpg");
				}

				if (imageL.FileName != String.Empty)
				{
					//jpgϸ Ѵ.
					if (imageS.FileName.Substring(imageS.FileName.Length-3).ToLower() != "jpg")
						throw new ArgumentException();

					imageL.SaveAs(imgPathFront + "L_" + strProdNum + ".jpg");
				}

				stTrans.Commit();
				result = 1;
			}
			catch(ArgumentException ae) 
			{
				string s = ae.Message;
				stTrans.Rollback();
				result = -2;
			}
			catch(Exception e) 
			{
				string s = e.Message;
				stTrans.Rollback();
				result = -1;
			}
			finally
			{
				myConnection.Close();
			}
			return result;
		}

		//Shop_AddShoppingCart
		/*
		@CartID nvarchar(50),
		@Quantity int,
		@ProductID int
		*/
		public int AddShoppingCart(string cartId, int quantity, int productid)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("Shop_AddShoppingCart", myConnection);

			myCommand.CommandType = CommandType.StoredProcedure;
			myCommand.Parameters.Add("RETURN_VALUE", SqlDbType.Int).Direction = ParameterDirection.ReturnValue;
			myCommand.Parameters.Add(new SqlParameter("@CartID", SqlDbType.NVarChar, 50)).Value = cartId;
			myCommand.Parameters.Add(new SqlParameter("@Quantity", SqlDbType.Int)).Value = quantity;
			myCommand.Parameters.Add(new SqlParameter("@ProductID", SqlDbType.Int)).Value = productid;

			myConnection.Open();

			int newId;
			try
			{
				myCommand.ExecuteNonQuery();
				newId = Convert.ToInt32(myCommand.Parameters["RETURN_VALUE"].Value);
			}
			catch(Exception e) 
			{
				string s = e.Message;
				newId = -1;
			}
			finally
			{
				myConnection.Close();
			}

			return newId;
		}

		/*
		Shop_AddShoppingCartOption
		@ShopCartID int,
		@OptionID int
		*/
		public int AddShoppingCartOption(int shopCartId, int optionId)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("Shop_AddShoppingCartOption", myConnection);

			myCommand.CommandType = CommandType.StoredProcedure;
			myCommand.Parameters.Add(new SqlParameter("@ShopCartID", SqlDbType.Int)).Value = shopCartId;
			myCommand.Parameters.Add(new SqlParameter("@OptionID", SqlDbType.Int)).Value = optionId;

			myConnection.Open();

			int result;
			try
			{
				myCommand.ExecuteNonQuery();

				result = 1;
			}
			catch(Exception e) 
			{
				string s = e.Message;
				result = -1;
			}
			finally
			{
				myConnection.Close();
			}

			return result;
		}


		public String GetShoppingCartId() 
		{

			// Obtain current HttpContext of ASP+ Request
			System.Web.HttpContext context = System.Web.HttpContext.Current;

			// If the user is authenticated, use their customerId as a permanent shopping cart id
			if (context.User.Identity.Name != "") 
			{
				return context.User.Identity.Name;
			}

			// If user is not authenticated, either fetch (or issue) a new temporary cartID
			if (context.Request.Cookies["Shop_CartID"] != null) 
			{
				return context.Request.Cookies["Shop_CartID"].Value;
			}
			else 
			{
				// Generate a new random GUID using System.Guid Class
				Guid tempCartId = Guid.NewGuid();

				// Send tempCartId back to client as a cookie
				context.Response.Cookies["Shop_CartID"].Value = tempCartId.ToString();

				// Return tempCartId
				return tempCartId.ToString();
			}
		}

		public SqlDataReader GetAltercastImageInfo(string kind)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("Shop_GetAltercastImageInfo", myConnection);
			myCommand.Parameters.Add(new SqlParameter("@Kind", SqlDbType.Char, 1)).Value = kind;

			myCommand.CommandType = CommandType.StoredProcedure;

			myConnection.Open();
			SqlDataReader dr = myCommand.ExecuteReader(CommandBehavior.CloseConnection);

			return dr;
		}

		public int UpdateAltercastGenerated(int prodId, string kind, string imageTail)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("Shop_UpdateAltercastGenerated", myConnection);

			myCommand.CommandType = CommandType.StoredProcedure;
			myCommand.Parameters.Add(new SqlParameter("@ID", SqlDbType.Int)).Value = prodId;
			myCommand.Parameters.Add(new SqlParameter("@Kind", SqlDbType.Char, 1)).Value = kind;
			myCommand.Parameters.Add(new SqlParameter("@ImageTail", SqlDbType.Char, 18)).Value = imageTail;

			myConnection.Open();

			int result;
			try
			{
				myCommand.ExecuteNonQuery();

				result = 1;
			}
			catch(Exception e) 
			{
				string s = e.Message;
				result = -1;
			}
			finally
			{
				myConnection.Close();
			}

			return result;
		}


		/*
		@kind char(1),
		@appendString varchar(50), 
		@stringPlace int,
		@stringColor varchar(30),
		@stringFont varchar(50),
		@stringSize int,
		@overImage varchar(50),
		@overCoordX int, 
		@overCoordY int,
		@overRotate int,
		@overAlpha int,
		@overPercent int,
		@overStringPosX int,
		@overStringPosY int,
		@overStringHAuto bit,
		@overStringVAuto bit,
		@originalSize int
		*/ 

		public int SaveAltercastImageInfo(string kind, string	appendString, 
			int stringPlace, string stringColor,	string stringFont, 
			int stringSize,	string overImage, int overCoordX, int overCoordY,
			int overRotate, int overAlpha, int overPercent, int overStringPosX, 
			int overStringPosY, bool overStringHAuto, bool overStringVAuto,
			int originalSize)

		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("Shop_SaveAltercastImageInfo", myConnection);

			myCommand.CommandType = CommandType.StoredProcedure;
			
			myCommand.Parameters.Add(new SqlParameter("@kind", SqlDbType.Char, 1)).Value = kind;
			myCommand.Parameters.Add(new SqlParameter("@appendString", SqlDbType.VarChar, 50)).Value = appendString;
			myCommand.Parameters.Add(new SqlParameter("@stringPlace", SqlDbType.Int)).Value = stringPlace;
			myCommand.Parameters.Add(new SqlParameter("@stringColor", SqlDbType.VarChar, 30)).Value = stringColor;
			myCommand.Parameters.Add(new SqlParameter("@stringFont", SqlDbType.VarChar, 50)).Value = stringFont;
			myCommand.Parameters.Add(new SqlParameter("@stringSize", SqlDbType.Int)).Value = stringSize;
			myCommand.Parameters.Add(new SqlParameter("@overImage", SqlDbType.VarChar, 50)).Value = overImage;
			myCommand.Parameters.Add(new SqlParameter("@overCoordX", SqlDbType.Int)).Value = overCoordX;
			myCommand.Parameters.Add(new SqlParameter("@overCoordY", SqlDbType.Int)).Value = overCoordY;
			myCommand.Parameters.Add(new SqlParameter("@overRotate", SqlDbType.Int)).Value = overRotate;
			myCommand.Parameters.Add(new SqlParameter("@overAlpha", SqlDbType.Int)).Value = overAlpha;
			myCommand.Parameters.Add(new SqlParameter("@overPercent", SqlDbType.Int)).Value = overPercent;
			myCommand.Parameters.Add(new SqlParameter("@overStringPosX", SqlDbType.Int)).Value = overStringPosX;
			myCommand.Parameters.Add(new SqlParameter("@overStringPosY", SqlDbType.Int)).Value = overStringPosY;
			myCommand.Parameters.Add(new SqlParameter("@overStringHAuto", SqlDbType.Bit)).Value = overStringHAuto;
			myCommand.Parameters.Add(new SqlParameter("@overStringVAuto", SqlDbType.Bit)).Value = overStringVAuto;
			myCommand.Parameters.Add(new SqlParameter("@originalSize", SqlDbType.Int)).Value = originalSize;

			myConnection.Open();

			int result;
			try
			{
				myCommand.ExecuteNonQuery();

				result = 1;
			}
			catch(Exception e) 
			{
				string s = e.Message;
				result = -1;
			}
			finally
			{
				myConnection.Close();
			}

			return result;
		}
	}
}
