using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;

namespace CustomControls.TextBox
{
	[
	DefaultProperty("Text"),
	ValidationProperty("Text"),
	ToolboxData("<{0}:SimpleTextBox runat=server></{0}:SimpleTextBox>")
	]
	public class SimpleTextBox : System.Web.UI.WebControls.WebControl, IPostBackDataHandler
	{
		public event EventHandler TextChanged;

		private bool isRunTime;

#region Ӽ 
		private string text;
		private bool autoPostBack;
		private int columns = 0;
		private int rows = 0;
		private TextBoxMode textMode = TextBoxMode.SingleLine;
#endregion

#region Ӽ

		[Bindable(true), 
			Category("Appearance"), 
			DefaultValue("")] 
		public string Text 
		{
			get	{return text;}
			set	{text = value;}
		}

		[
		Bindable(true), 
		Category("Behavior"), 
		DefaultValue(false)
		]
		public bool AutoPostBack
		{
			get	{return autoPostBack;}
			set	{autoPostBack = value;}
		}

		[
		Bindable(true), 
		Category("Appearance"), 
		DefaultValue(0)
		]
		public int Columns
		{
			get	{return columns;}
			set	{columns = value;}
		}

		[
		Bindable(true), 
		Category("Appearance"), 
		DefaultValue(0)
		]
		public int Rows
		{
			get	{return rows;}
			set	{rows = value;}
		}

		[
		Bindable(true), 
		Category("Appearance"), 
		DefaultValue(TextBoxMode.SingleLine)
		]
		public TextBoxMode TextMode
		{
			get	{return textMode;}
			set	{textMode = value;}
		}

#endregion

#region ޼ҵ

		public SimpleTextBox()// : base(HtmlTextWriterTag.Input)
		{
		}

		public bool LoadPostData(string postDataKey, 
			System.Collections.Specialized.NameValueCollection postCollection)
		{
			string newVal = postCollection[postDataKey];

			if(text != newVal)
			{
				text = newVal;
				return true;
			}

			return false;
		}

		public void RaisePostDataChangedEvent()
		{
			if (TextChanged != null)
				TextChanged(this, EventArgs.Empty);
		}

		protected override void AddAttributesToRender(System.Web.UI.HtmlTextWriter writer)
		{
			writer.AddAttribute(HtmlTextWriterAttribute.Name, UniqueID);
			switch (textMode)
			{
				case TextBoxMode.SingleLine :
					writer.AddAttribute(HtmlTextWriterAttribute.Type, "input");
					writer.AddAttribute(HtmlTextWriterAttribute.Value, text);
					break;
				case TextBoxMode.Password :
					writer.AddAttribute(HtmlTextWriterAttribute.Type, "password");
					writer.AddAttribute(HtmlTextWriterAttribute.Value, text);
					break;
				case TextBoxMode.MultiLine :
					if (rows > 0)
						writer.AddAttribute(HtmlTextWriterAttribute.Rows, rows.ToString());
					break;
			}
			if (columns > 0)
				writer.AddAttribute(HtmlTextWriterAttribute.Cols, columns.ToString());

			// Ÿӿ  ʵѴ.
			// Ÿӿ Ⱑ Ǹ Request ü 
			//    Ʈ  ̻ϰ δ.
			if (isRunTime)
			{
				//W3C XML Doom  1̻̰ ECMAScript  1.2 ̻
				bool scriptEnable = Page.Request.Browser.W3CDomVersion.Major >= 1 && 
					Page.Request.Browser.EcmaScriptVersion.CompareTo(new Version(1, 2)) >= 0;
				if (autoPostBack && scriptEnable)
					writer.AddAttribute(HtmlTextWriterAttribute.Onchange, Page.GetPostBackClientEvent(this, ""));
			}
				
			base.AddAttributesToRender(writer);
		}
		
		protected override void RenderContents(System.Web.UI.HtmlTextWriter writer)
		{
			if (textMode == TextBoxMode.MultiLine)
				writer.Write(text);

			base.RenderContents(writer);
		}

		public override void RenderBeginTag(System.Web.UI.HtmlTextWriter writer)
		{
			/**************************************************
			 * RenderBeginTag ޼ҵ带    
			 * base.RenderBeginTag ȣ 
			 * AddAttributesToRender ޼尡 ȣ ʴ´.
			 * ׷ AddAttributesToRender  ȣѴ.
			***************************************************/
			AddAttributesToRender(writer);

			switch (textMode)
			{
				case TextBoxMode.SingleLine :
					writer.RenderBeginTag(HtmlTextWriterTag.Input);
					break;
				case TextBoxMode.Password :
					writer.RenderBeginTag(HtmlTextWriterTag.Input);
					break;
				case TextBoxMode.MultiLine :
					writer.RenderBeginTag(HtmlTextWriterTag.Textarea);
					break;
			}
		}

		public override void RenderEndTag(System.Web.UI.HtmlTextWriter writer)
		{
			writer.RenderEndTag();
		}

		protected override void OnPreRender(System.EventArgs e)
		{
			// Ÿӿ  ޼ҵ尡 ȣ ʴ´.
			isRunTime = true;
			base.OnPreRender(e);
		}

		protected override void LoadViewState(object savedState)
		{
			if (savedState != null)
			{
				object[] stateArr = (object[])savedState;
				if (stateArr[0] != null)
					text = stateArr[0].ToString();
				autoPostBack = (bool)stateArr[1];
				columns = (int)stateArr[2];
				rows = (int)stateArr[3];
				textMode = (TextBoxMode)stateArr[4];
			}
		}

		protected override object SaveViewState()
		{
			object[] stateArr = new object[5];
			stateArr[0] = text;
			stateArr[1] = autoPostBack;
			stateArr[2] = columns;
			stateArr[3] = rows;
			stateArr[4] = textMode;

			return (object)stateArr;
		}
#endregion
	}
}
