using System;
using System.Drawing;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;

namespace CustomControls.Validators
{
	[DefaultProperty("ErrorMessage")]
	public abstract class SimpleBaseValidator1 : System.Web.UI.WebControls.Label, IValidator
	{
		private bool isValid = true;
		
		//DefaultValueAttribute  ϱ 
		[DefaultValue(typeof(Color), "Red")]                                         
		public override Color ForeColor {
			get {return base.ForeColor;}
			set {base.ForeColor = value;}
		}

		[Category("Behavior"),
		DefaultValue(""),
		TypeConverter(typeof(ValidatedControlConverter))]                                         
		public string ControlToValidate {
			get { 
				object o = ViewState["ControlToValidate"];
				return((o == null) ? String.Empty : (string)o);
			}
			set {ViewState["ControlToValidate"] = value;}
		}

		[Category("Appearance"),
		DefaultValue("")]
		public string ErrorMessage {
			get {return base.Text;}
			set {base.Text = value;}
		}

		[Browsable(false),
		DefaultValue(true),
		DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public bool IsValid {
			get {return isValid;}
			set {isValid = value;}
		}

		protected string GetControlValidationValue(string name) 
		{
			Control c = NamingContainer.FindControl(name);            
			if (c == null) 
				return null;

			PropertyDescriptor prop = GetValidationProperty(c);
			if (prop == null) 
				return null;

			object value = prop.GetValue(c);
			if (value != null) 
				return value.ToString();
			else 
				return string.Empty;
		}                

		public static PropertyDescriptor GetValidationProperty(object component) 
		{
			ValidationPropertyAttribute valProp = (ValidationPropertyAttribute)TypeDescriptor.GetAttributes(component)[typeof(ValidationPropertyAttribute)];
			if (valProp != null && valProp.Name != null) 
				return TypeDescriptor.GetProperties(component, null)[valProp.Name];

			return null;
		}

		protected override void OnInit(EventArgs e) 
		{
			base.OnInit(e);

			// ޼ 
			ForeColor = Color.Red;
			Page.Validators.Add(this);
		}        

		protected override void OnUnload(EventArgs e) 
		{
			if (Page != null) 
				Page.Validators.Remove(this);

			base.OnUnload(e);
		}
		
		protected override void Render(HtmlTextWriter writer)
		{
			if (!IsValid)
				base.Render(writer);
		}

		//ڽ Ʈѿ ؾϴ ߻ Ŭ
		protected abstract bool EvaluateIsValid();

		public void Validate() 
		{
			//Visible false  ˻ 
			if (!Visible) {
				IsValid = true;
				return;
			}

			// Ʈ ϴ  θ Ʈ 
			//ϳ Visible false  ˻ 
			Control parent = Parent;
			while (parent != null) {
				if (!parent.Visible) {
					IsValid = true;
					return;
				}
				parent = parent.Parent;
			}

			IsValid = EvaluateIsValid();
		}  
	}
}