using System;
using System.IO;
using System.Web;
using System.Data;
using System.Data.SqlClient;
using System.Configuration;

namespace PowerOfDotNet.Components
{
	public class ClubInfo
	{
		public string subject;
		public string sysopId;
		public string viceSysopId;
		public string sysopName;
		public string sysopEmail;
		public int memberCount;
		public DateTime CreatedDate;
		public string frontTableName;
		public string introHtml;
		public string mainHtml;
		public byte skinType;
		public int classId;
		public byte publicType;
		public string className;
		public string classImagePath;
	}

	/// <summary>
	/// ClubsDB   Դϴ.
	/// </summary>
	public class ClubsDB
	{
		public ClubsDB()
		{
		}

		public void UserVisitedCountIncrease(int clubid, string userId)
		{
			//Club_UserVisitedCountIncrease
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("Club_UserVisitedCountIncrease", myConnection);

			myCommand.CommandType = CommandType.StoredProcedure;
			myCommand.Parameters.Add(new SqlParameter("@UserID", SqlDbType.VarChar, 12)).Value = userId;
			myCommand.Parameters.Add(new SqlParameter("@ClubID", SqlDbType.Int)).Value = clubid;

			myConnection.Open();
			myCommand.ExecuteNonQuery();
			myConnection.Close();
		}

		public SqlDataReader GetUserClubs(string userId)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("Club_GetUserClubs", myConnection);

			myCommand.CommandType = CommandType.StoredProcedure;
			myCommand.Parameters.Add(new SqlParameter("@UserID", SqlDbType.VarChar, 12)).Value = userId;

			myConnection.Open();
			SqlDataReader dr = myCommand.ExecuteReader(CommandBehavior.CloseConnection);

			return dr;
		}

		/*
		public SqlDataReader GetBoardBySubject(int clubId, string subject)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("Club_GetBoardBySubject", myConnection);

			myCommand.CommandType = CommandType.StoredProcedure;
			myCommand.Parameters.Add(new SqlParameter("@ClubID", SqlDbType.Int)).Value = clubId;
			myCommand.Parameters.Add(new SqlParameter("@Subject", SqlDbType.NVarChar, 50)).Value = subject;

			myConnection.Open();
			SqlDataReader dr = myCommand.ExecuteReader(CommandBehavior.CloseConnection);

			return dr;
		}
		*/

		public SqlDataReader GetBoardBySortOrder(int clubId, int sortOrder)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("Club_GetBoardBySortOrder", myConnection);

			myCommand.CommandType = CommandType.StoredProcedure;
			myCommand.Parameters.Add(new SqlParameter("@ClubID", SqlDbType.Int)).Value = clubId;
			myCommand.Parameters.Add(new SqlParameter("@SortOrder", SqlDbType.Int)).Value = sortOrder;

			myConnection.Open();
			SqlDataReader dr = myCommand.ExecuteReader(CommandBehavior.CloseConnection);

			return dr;
		}

		public SqlDataReader GetBoardListUsingTop(string tableName, int top)
		{
			string sql = "select top " + top +" * from " + tableName + " order by linkid desc";

			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand(sql, myConnection);

			myConnection.Open();
			SqlDataReader dr = myCommand.ExecuteReader(CommandBehavior.CloseConnection);

			return dr;
		}

		public int WithdrawalFromClub(int clubId, string userId)
		{
			//Club_WithdrawalFromClub
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("Club_WithdrawalFromClub", myConnection);

			myCommand.CommandType = CommandType.StoredProcedure;
			myCommand.Parameters.Add("RETURN_VALUE", SqlDbType.Int).Direction = ParameterDirection.ReturnValue;
			myCommand.Parameters.Add(new SqlParameter("@ClubID", SqlDbType.Int)).Value = clubId;
			myCommand.Parameters.Add(new SqlParameter("@UserID", SqlDbType.VarChar, 12)).Value = userId;

			myConnection.Open();
			int result;
			try 
			{
				myCommand.ExecuteNonQuery();
				result = Convert.ToInt32(myCommand.Parameters["RETURN_VALUE"].Value);
			}
			catch
			{
				result = -1;
			}
			finally
			{
				myConnection.Close();
			}

			return result;

		}

		public SqlDataReader GetBoardList(int clubId)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("Club_GetBoardList", myConnection);

			myCommand.CommandType = CommandType.StoredProcedure;
			myCommand.Parameters.Add(new SqlParameter("@ClubID", SqlDbType.Int)).Value = clubId;

			myConnection.Open();
			SqlDataReader dr = myCommand.ExecuteReader(CommandBehavior.CloseConnection);

			return dr;
		}

		public int UserAgreement(int clubId, string userId)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("Club_UserAgreement", myConnection);

			myCommand.CommandType = CommandType.StoredProcedure;
			myCommand.Parameters.Add(new SqlParameter("@ClubID", SqlDbType.Int)).Value = clubId;
			myCommand.Parameters.Add(new SqlParameter("@UserID", SqlDbType.VarChar, 12)).Value = userId;

			myConnection.Open();
			int result = 1;
			try 
			{
				myCommand.ExecuteNonQuery();
			}
			catch
			{
				result = -1;
			}
			finally
			{
				myConnection.Close();
			}

			return result;

		}

		public int UpdateClub(int clubId, string subject, int classId, byte publicType, 
							string sysopId, string viceSysopIDs, string introHtml, 
							string mainHtml, byte skinType)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("Club_UpdateClub", myConnection);

			myCommand.CommandType = CommandType.StoredProcedure;
			myCommand.Parameters.Add(new SqlParameter("@ID", SqlDbType.Int)).Value = clubId;
			myCommand.Parameters.Add(new SqlParameter("@Subject", SqlDbType.NVarChar, 50)).Value = subject;
			myCommand.Parameters.Add(new SqlParameter("@ClassID", SqlDbType.Int)).Value = classId;
			myCommand.Parameters.Add(new SqlParameter("@PublicType", SqlDbType.TinyInt)).Value = publicType;
			myCommand.Parameters.Add(new SqlParameter("@SysopID", SqlDbType.VarChar, 12)).Value = sysopId;
			myCommand.Parameters.Add(new SqlParameter("@ViceSysopIDs", SqlDbType.NVarChar, 50)).Value = viceSysopIDs;
			myCommand.Parameters.Add(new SqlParameter("@IntroHtml", SqlDbType.NVarChar, 500)).Value = introHtml;
			myCommand.Parameters.Add(new SqlParameter("@MainHtml", SqlDbType.NText)).Value = mainHtml;
			myCommand.Parameters.Add(new SqlParameter("@SkinType", SqlDbType.TinyInt)).Value = skinType;

			int result = -1;

			myConnection.Open();
			try
			{
				myCommand.ExecuteNonQuery();
				result = 1;
			}
			catch (Exception err)
			{
				string s = err.Message;
			}
			finally
			{
				myConnection.Close();
			}

			return result;
		}


		public int CheckClubUser(int clubId, string userId)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("Club_CheckClubUser", myConnection);

			myCommand.CommandType = CommandType.StoredProcedure;
			myCommand.Parameters.Add("RETURN_VALUE", SqlDbType.Int).Direction = ParameterDirection.ReturnValue;
			myCommand.Parameters.Add(new SqlParameter("@ClubID", SqlDbType.Int)).Value = clubId;
			myCommand.Parameters.Add(new SqlParameter("@UserID", SqlDbType.VarChar, 12)).Value = userId;

			myConnection.Open();
			myCommand.ExecuteNonQuery();
			int result = Convert.ToInt32(myCommand.Parameters["RETURN_VALUE"].Value);
			myConnection.Close();

			//-1 : ȸ, 0 :   ȵ, 1 : ȸ
			return result;
		}

		public SqlDataReader GetClub(int clubId)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("Club_GetClub", myConnection);

			myCommand.CommandType = CommandType.StoredProcedure;
			myCommand.Parameters.Add(new SqlParameter("@ClubID", SqlDbType.Int)).Value = clubId;

			myConnection.Open();
			SqlDataReader dr = myCommand.ExecuteReader(CommandBehavior.CloseConnection);

			return dr;
		}

		public int JoinClubUser(int clubId, string userId, string selfIntroduction, string openInformations, byte publicType)
		{
			//publicType ٸ óؾ Ѵ.

			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("Club_JoinClubUser", myConnection);

			myCommand.CommandType = CommandType.StoredProcedure;
			myCommand.Parameters.Add("RETURN_VALUE", SqlDbType.Int).Direction = ParameterDirection.ReturnValue;
			myCommand.Parameters.Add(new SqlParameter("@ClubID", SqlDbType.Int)).Value = clubId;
			myCommand.Parameters.Add(new SqlParameter("@UserID", SqlDbType.VarChar, 12)).Value = userId;
			myCommand.Parameters.Add(new SqlParameter("@SelfIntroduction", SqlDbType.VarChar, 300)).Value = selfIntroduction;
			myCommand.Parameters.Add(new SqlParameter("@OpenInformations", SqlDbType.VarChar, 200)).Value = openInformations;
			myCommand.Parameters.Add(new SqlParameter("@Agreement", SqlDbType.Bit)).Value = publicType != 2;

			int newClubUser = -1;

			myConnection.Open();
			try
			{
				myCommand.ExecuteNonQuery();
				newClubUser = Convert.ToInt32(myCommand.Parameters["RETURN_VALUE"].Value);
			}
			catch (Exception err)
			{
				string s = err.Message;
			}
			finally
			{
				myConnection.Close();
			}

			return newClubUser;
		}

		public DataSet GetClubUserList(int clubId)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlDataAdapter da = new SqlDataAdapter("Club_GetClubUserList", myConnection);

			da.SelectCommand.CommandType = CommandType.StoredProcedure;
			da.SelectCommand.Parameters.Add(new SqlParameter("@ClubID", SqlDbType.Int)).Value = clubId;

			DataSet ds = new DataSet();
			da.Fill(ds, "ClubUserList");

			return ds;
		}

		public SqlDataReader GetClubUserInfo(int clubId, string userId)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("Club_GetClubUserInfo", myConnection);

			myCommand.CommandType = CommandType.StoredProcedure;
			myCommand.Parameters.Add(new SqlParameter("@ClubID", SqlDbType.Int)).Value = clubId;
			myCommand.Parameters.Add(new SqlParameter("@UserID", SqlDbType.VarChar, 12)).Value = userId;

			myConnection.Open();
			SqlDataReader dr = myCommand.ExecuteReader(CommandBehavior.CloseConnection);

			return dr;
		}

		public DataSet GetWaitingUsers(int clubId)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlDataAdapter da = new SqlDataAdapter("Club_GetWaitingUsers", myConnection);

			da.SelectCommand.CommandType = CommandType.StoredProcedure;
			da.SelectCommand.Parameters.Add(new SqlParameter("@ClubID", SqlDbType.Int)).Value = clubId;

			DataSet ds = new DataSet();
			da.Fill(ds, "ClubUserList");

			return ds;
		}

		public DataSet GetClubListWhereSubject(string subject)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlDataAdapter da = new SqlDataAdapter("Club_GetClubListWhereSubject", myConnection);

			da.SelectCommand.CommandType = CommandType.StoredProcedure;
			da.SelectCommand.Parameters.Add(new SqlParameter("@Subject", SqlDbType.NVarChar, 50)).Value = subject;

			DataSet ds = new DataSet();
			da.Fill(ds, "ClubUserList");

			return ds;
		}


		public DataSet GetClubListWhereSysopId(string sysopId)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlDataAdapter da = new SqlDataAdapter("Club_GetClubListWhereSysopId", myConnection);

			da.SelectCommand.CommandType = CommandType.StoredProcedure;
			da.SelectCommand.Parameters.Add(new SqlParameter("@SysopID", SqlDbType.VarChar, 12)).Value = sysopId;

			DataSet ds = new DataSet();
			da.Fill(ds, "ClubUserList");

			return ds;
		}


		public DataSet GetClubListWhereIntroHtml(string introHtml)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlDataAdapter da = new SqlDataAdapter("Club_GetClubListWhereIntroHtml", myConnection);

			da.SelectCommand.CommandType = CommandType.StoredProcedure;
			da.SelectCommand.Parameters.Add(new SqlParameter("@IntroHtml", SqlDbType.NVarChar, 50)).Value = introHtml;

			DataSet ds = new DataSet();
			da.Fill(ds, "ClubUserList");

			return ds;
		}

		public int UpdateMemberInfo(int clubId, string userId, string selfIntroduction, string openInformations)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("Club_UpdateMemberInfo", myConnection);

			myCommand.CommandType = CommandType.StoredProcedure;

			myCommand.Parameters.Add(new SqlParameter("@ClubID", SqlDbType.Int)).Value = clubId;
			myCommand.Parameters.Add(new SqlParameter("@UserID", SqlDbType.VarChar, 12)).Value = userId;
			myCommand.Parameters.Add(new SqlParameter("@SelfIntroduction", SqlDbType.VarChar, 300)).Value = selfIntroduction;
			myCommand.Parameters.Add(new SqlParameter("@OpenInformations", SqlDbType.VarChar, 200)).Value = openInformations;

			int result;

			myConnection.Open();
			try
			{
				myCommand.ExecuteNonQuery();
				result = 1;
			}
			catch (Exception err)
			{
				string s = err.Message;
				result = -1;
			}
			finally
			{
				myConnection.Close();
			}

			return result;
		}


		public int UpdateBoardNameAndType(int boardId, string subject, byte type)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand("Club_UpdateBoardNameAndType", myConnection);

			myCommand.CommandType = CommandType.StoredProcedure;

			myCommand.Parameters.Add(new SqlParameter("@ID", SqlDbType.Int)).Value = boardId;
			myCommand.Parameters.Add(new SqlParameter("@Subject", SqlDbType.NVarChar, 50)).Value = subject;
			myCommand.Parameters.Add(new SqlParameter("@Type", SqlDbType.TinyInt)).Value = type;

			int result;

			myConnection.Open();
			try
			{
				myCommand.ExecuteNonQuery();
				result = 1;
			}
			catch (Exception err)
			{
				string s = err.Message;
				result = -1;
			}
			finally
			{
				myConnection.Close();
			}

			return result;
		}

		public int RemoveClub(int clubId)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand getClubCommand = new SqlCommand("Club_GetClub", myConnection);

			getClubCommand.CommandType = CommandType.StoredProcedure;

			getClubCommand.Parameters.Add(new SqlParameter("@ClubID", SqlDbType.Int)).Value = clubId;

			myConnection.Open();
			SqlDataReader dr = getClubCommand.ExecuteReader();
			dr.Read();
			string tableNameFront = dr["FrontTableName"].ToString();
			dr.Close();

			SqlCommand removeClubBoardCommand = new SqlCommand("Club_RemoveClubBoard", myConnection);
			removeClubBoardCommand.CommandType = CommandType.StoredProcedure;
			removeClubBoardCommand.Parameters.Add(new SqlParameter("@TableName", SqlDbType.Int)).Value = clubId;

			SqlCommand deleteClubCommand = new SqlCommand("Club_DeleteClub", myConnection);
			deleteClubCommand.CommandType = CommandType.StoredProcedure;
			deleteClubCommand.Parameters.Add(new SqlParameter("@ClubID", SqlDbType.Int)).Value = clubId;

			SqlTransaction stTrans = myConnection.BeginTransaction();
			//Ʈ
			removeClubBoardCommand.Transaction = stTrans;
			deleteClubCommand.Transaction = stTrans;

			int result;

			try
			{
				deleteClubCommand.ExecuteNonQuery();

				//ũ ̺ 5
				for (int i=0;i<5; i++)
				{
					string strBoardNum = "0" + (i + 1);
					string boardTableName = tableNameFront + strBoardNum.Substring(strBoardNum.Length-2);

					removeClubBoardCommand.Parameters.Clear();
					removeClubBoardCommand.Parameters.Add(new SqlParameter("@TableName", SqlDbType.VarChar, 50)).Value = boardTableName;

					removeClubBoardCommand.ExecuteNonQuery();
				}

				string clubDirPath = HttpContext.Current.Server.MapPath("/Clubs/ClubsDir/clubId_" + clubId + "/");
				if (Directory.Exists(clubDirPath))
				{
					string[] aFiles = Directory.GetFiles(clubDirPath);
					foreach (string fileName in aFiles)
						File.Delete(fileName);

					Directory.Delete(clubDirPath);
				}

				stTrans.Commit();
				result = 1;
			}
			catch (Exception err)
			{
				stTrans.Rollback();
				string s = err.Message;
				result = -1;
			}
			finally
			{
				myConnection.Close();
			}

			return result;
		}


		public DataSet GetClubList(int classId)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlDataAdapter da = new SqlDataAdapter("Club_GetClubList", myConnection);

			da.SelectCommand.CommandType = CommandType.StoredProcedure;
			da.SelectCommand.Parameters.Add(new SqlParameter("@ClassID", SqlDbType.Int)).Value = classId;

			DataSet ds = new DataSet();
			da.Fill(ds, "ClubList");

			return ds;
		}

		public DataSet GetClubListOrderByMemberCount()
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlDataAdapter da = new SqlDataAdapter("Club_GetClubListOrderByMemberCount", myConnection);

			da.SelectCommand.CommandType = CommandType.StoredProcedure;
			
			DataSet ds = new DataSet();
			da.Fill(ds, "ClubList");

			return ds;
		}

		public DataSet GetClubListOrderByCreatedDate()
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlDataAdapter da = new SqlDataAdapter("Club_GetClubListOrderByCreatedDate", myConnection);

			da.SelectCommand.CommandType = CommandType.StoredProcedure;
			
			DataSet ds = new DataSet();
			da.Fill(ds, "ClubList");

			return ds;
		}

		public SqlDataReader GetClubsClass()
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand getClubClassCmd = new SqlCommand("Club_GetClassList", myConnection);

			getClubClassCmd.CommandType = CommandType.StoredProcedure;

			myConnection.Open();
			SqlDataReader dr = getClubClassCmd.ExecuteReader(CommandBehavior.CloseConnection);

			return dr;
		}

		public SqlDataReader GetClass(int classId)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand getClubClassCmd = new SqlCommand("Club_GetClass", myConnection);

			getClubClassCmd.CommandType = CommandType.StoredProcedure;
			getClubClassCmd.Parameters.Add(new SqlParameter("@ID", SqlDbType.Int)).Value = classId;

			myConnection.Open();
			SqlDataReader dr = getClubClassCmd.ExecuteReader(CommandBehavior.CloseConnection);

			return dr;
		}

		public SqlDataReader GetClubAnnouncementRecent(int topCount)
		{
			string sql = "select top " + topCount + " ID, GroupID, Writer, Subject, RegDate from ClubMainAnnouncement01 order by [Order] desc";
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand myCommand = new SqlCommand(sql, myConnection);

			myConnection.Open();
			SqlDataReader dr = myCommand.ExecuteReader(CommandBehavior.CloseConnection);

			return dr;
		}

		public int CreateNewClub(string subject, int classId, byte publicType, string sysopId, string IntroHtml)
		{
			SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
			SqlCommand addNewClubInfoCommand = new SqlCommand("Club_NewClubInfoAdd", myConnection);

			addNewClubInfoCommand.CommandType = CommandType.StoredProcedure;

			addNewClubInfoCommand.Parameters.Add("RETURN_VALUE", SqlDbType.Int).Direction = ParameterDirection.ReturnValue;
			addNewClubInfoCommand.Parameters.Add(new SqlParameter("@Subject", SqlDbType.NVarChar, 50)).Value = subject;
			addNewClubInfoCommand.Parameters.Add(new SqlParameter("@ClassID", SqlDbType.Int)).Value = classId;
			addNewClubInfoCommand.Parameters.Add(new SqlParameter("@PublicType", SqlDbType.TinyInt)).Value = publicType;
			addNewClubInfoCommand.Parameters.Add(new SqlParameter("@SysopID", SqlDbType.VarChar, 12)).Value = sysopId;
			addNewClubInfoCommand.Parameters.Add(new SqlParameter("@IntroHtml", SqlDbType.VarChar, 500)).Value = IntroHtml;
			SqlParameter sqlParmFrontTableName = new SqlParameter("@FrontTableName", SqlDbType.VarChar, 30);
			sqlParmFrontTableName.Direction = ParameterDirection.Output;
			addNewClubInfoCommand.Parameters.Add(sqlParmFrontTableName);

			SqlCommand addBoardInfoCommand = new SqlCommand("Club_BoardInfoAdd", myConnection);
			addBoardInfoCommand.CommandType = CommandType.StoredProcedure;

			SqlCommand createBoardTableCommand = new SqlCommand("Club_CreateBoardTable", myConnection);
			createBoardTableCommand.CommandType = CommandType.StoredProcedure;

			SqlCommand userJoinCommand = new SqlCommand("Club_UserJoin", myConnection);
			userJoinCommand.CommandType = CommandType.StoredProcedure;

			myConnection.Open();

			SqlTransaction stTrans = myConnection.BeginTransaction();
			//Ʈ
			addNewClubInfoCommand.Transaction = stTrans;
			addBoardInfoCommand.Transaction = stTrans;
			createBoardTableCommand.Transaction = stTrans;
			userJoinCommand.Transaction = stTrans;

			int newClubId = -1;
			try
			{
				addNewClubInfoCommand.ExecuteNonQuery();
				newClubId = Convert.ToInt32(addNewClubInfoCommand.Parameters["RETURN_VALUE"].Value);
				string boardTableNameHead = sqlParmFrontTableName.Value.ToString();

				string clubPageNameList = ConfigurationSettings.AppSettings["ClubPageNameList"];
				string clubPageNameWrite = ConfigurationSettings.AppSettings["ClubPageNameWrite"];
				string clubPageNameEdit = ConfigurationSettings.AppSettings["ClubPageNameEdit"];
				string clubPageNameReply = ConfigurationSettings.AppSettings["ClubPageNameReply"];
				string clubPageNameContent = ConfigurationSettings.AppSettings["ClubPageNameContent"];

				//ũ ̺ 5 ش.
				for (int i=0;i<5; i++)
				{
					string strBoardNum = "0" + (i + 1);
					string boardTableName = boardTableNameHead + strBoardNum.Substring(strBoardNum.Length-2);

					addBoardInfoCommand.Parameters.Clear();
					
					addBoardInfoCommand.Parameters.Add(new SqlParameter("@ClubID", SqlDbType.VarChar, 50)).Value = newClubId;
					addBoardInfoCommand.Parameters.Add(new SqlParameter("@Subject", SqlDbType.NVarChar, 12)).Value = "Խ" + (i+1);
					addBoardInfoCommand.Parameters.Add(new SqlParameter("@TableName", SqlDbType.NVarChar, 50)).Value = boardTableName;
					addBoardInfoCommand.Parameters.Add(new SqlParameter("@Html", SqlDbType.NText)).Value = "";
					addBoardInfoCommand.Parameters.Add(new SqlParameter("@DisplayType", SqlDbType.TinyInt)).Value = 0;
					addBoardInfoCommand.Parameters.Add(new SqlParameter("@Image", SqlDbType.VarChar, 80)).Value = "";
					//addBoardInfoCommand.Parameters.Add(new SqlParameter("@Roles", SqlDbType.NVarChar, 50)).Value = "";
					addBoardInfoCommand.Parameters.Add(new SqlParameter("@ImageHeadName", SqlDbType.VarChar, 40)).Value = "";
					addBoardInfoCommand.Parameters.Add(new SqlParameter("@CssClassHead", SqlDbType.VarChar, 20)).Value = "";
					addBoardInfoCommand.Parameters.Add(new SqlParameter("@Type", SqlDbType.TinyInt)).Value = 0;
					addBoardInfoCommand.Parameters.Add(new SqlParameter("@PageNameList", SqlDbType.NVarChar, 50)).Value = clubPageNameList;
					addBoardInfoCommand.Parameters.Add(new SqlParameter("@PageNameWrite", SqlDbType.NVarChar, 50)).Value = clubPageNameWrite;
					addBoardInfoCommand.Parameters.Add(new SqlParameter("@PageNameEdit", SqlDbType.NVarChar, 50)).Value = clubPageNameEdit;
					addBoardInfoCommand.Parameters.Add(new SqlParameter("@PageNameReply", SqlDbType.NVarChar, 50)).Value = clubPageNameReply;
					addBoardInfoCommand.Parameters.Add(new SqlParameter("@PageNameContent", SqlDbType.NVarChar, 12)).Value = clubPageNameContent;

					addBoardInfoCommand.Parameters.Add(new SqlParameter("@SortOrder", SqlDbType.Int)).Value = i;
					addBoardInfoCommand.ExecuteNonQuery();

					createBoardTableCommand.Parameters.Clear();
					createBoardTableCommand.Parameters.Add(new SqlParameter("@TableName", SqlDbType.NVarChar, 50)).Value = boardTableName;
					createBoardTableCommand.ExecuteNonQuery();
				}

				userJoinCommand.Parameters.Add(new SqlParameter("@ClubID", SqlDbType.Int)).Value = newClubId;
				userJoinCommand.Parameters.Add(new SqlParameter("@UserID", SqlDbType.VarChar,12)).Value = sysopId;
				userJoinCommand.Parameters.Add(new SqlParameter("@SelfIntroduction", SqlDbType.VarChar,300)).Value = "";
				userJoinCommand.Parameters.Add(new SqlParameter("@OpenInformations", SqlDbType.VarChar, 200)).Value = "";
				userJoinCommand.Parameters.Add(new SqlParameter("@Agreement", SqlDbType.Bit)).Value = 1;

				userJoinCommand.ExecuteNonQuery();

				string clubDirPath = HttpContext.Current.Server.MapPath("/Clubs/ClubsDir/clubId_" + newClubId + "/");
				if (!Directory.Exists(clubDirPath))
					Directory.CreateDirectory(clubDirPath);

				stTrans.Commit();
			}
			catch (Exception err)
			{
				string s = err.Message;
				stTrans.Rollback();
				newClubId = -2;
			}
			finally
			{
				myConnection.Close();
			}

			return newClubId;
		}
	}
}
