using System;
using System.IO;
using System.ComponentModel;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.Caching;
using System.Data;
using System.Data.SqlClient;
using System.Web.UI.HtmlControls;

namespace PowerOfDotNet {

    //*********************************************************************
    //
    // PortalModuleControl Class
    //
    // The PortalModuleControl class defines a custom base class inherited by all
    // desktop portal modules within the Portal.
    // 
    // The PortalModuleControl class defines portal specific properties
    // that are used by the portal framework to correctly display portal modules
    //
    //*********************************************************************
	[ToolboxData("<{0}:PortalModuleControl runat=server></{0}:PortalModuleControl>")]
    public class PortalModuleControl : UserControl {

        private ModuleSettings  _moduleConfiguration;
        private int             _isEditable = 0;
        private int             _portalId = 0;
        private Hashtable       _settings;

		//  ID
        [Browsable(false),DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public int ModuleId {
            get {
                return (int) _moduleConfiguration.ModuleId;
            }
        }    

		// Ż ID
        [Browsable(false),DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public int PortalId {
            get {
                return _portalId;
            }
            set {
                _portalId = value;
            }
        }
        
		//ڰ      
        [Browsable(false),DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public bool IsEditable {
            get {
                // Perform tri-state switch check to avoid having to perform a security
                // role lookup on every property access (instead caching the result)
                if (_isEditable == 0) {
                    PortalSettings portalSettings = (PortalSettings) HttpContext.Current.Items["PortalSettings"];

                    if (portalSettings.AlwaysShowEditButton == true || 
						PortalSecurity.IsInRoles(_moduleConfiguration.AuthorizedEditRoles)) {
                        _isEditable = 1;
                    }
                    else {
                        _isEditable = 2;
                    }
                }

                return (_isEditable == 1);
            }
        }

		//    ϴ ModuleSettings ü
        [Browsable(false),DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public ModuleSettings ModuleConfiguration {
            get {
                return _moduleConfiguration;
            }
            set {
                _moduleConfiguration = value;
            }
        }

		// ModuleSettings ̺ μ ⼳ ´.
        [Browsable(false),DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public Hashtable Settings {
            get {
                if (_settings == null) {

                    _settings = PortalSettings.GetModuleSettings(ModuleId);
                }

                return _settings;
            }
         }
    }
    

	/// <summary>
	/// PortalModuleControl  Ȯ Ŭ
	/// </summary>
	[ToolboxData("<{0}:PortalModuleControlEx runat=server></{0}:PortalModuleControlEx>")]
	public class PortalModuleControlEx : PortalModuleControl 
	{
		private Hashtable		_allPagesInfo;

		//ɽÿ     ⿡   Ű ´.
		[Browsable(false),DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public String CacheKey {
			get {
				return "UniqueCacheKey:" + this.ModuleId + this.GetType().ToString();
			}
		}

		//  Formü 
		[Browsable(false),DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public HtmlForm Form {
			get {
				foreach (Control oc in Page.Controls) {
					if (oc is HtmlForm)
						return (HtmlForm)oc;
				}

				throw new Exception("̻ϴ.   ?");
			}
		}

		//    ID  ID ϴ 
		//Hashtable ü ȯѴ.
		[Browsable(false),DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public Hashtable AllPagesInfo {
			get {
				if (_allPagesInfo == null)	{
					//߱Ⱓ   ȭ .
					//Hashtable oTmp = null;
					Hashtable oTmp = HttpContext.Current.Cache["_AllPagesInfo"] as Hashtable;
					
					if (oTmp != null) {
						//ĳÿ ̹  
						//͸ Ѵ.
						_allPagesInfo = oTmp;
					}
					else {
						// ĳÿ  ʾ ͺ̽ 
						//   ǰ  ID 
						//  ϱ  ĳÿ Ѵ.

						_allPagesInfo = new Hashtable();

						//ǰ  ID  ´.
						AdminDB db = new AdminDB();
						SqlDataReader dr = db.GetPages();

						while (dr.Read()) {
							int[] aPageInfo = new int[2] { dr.GetInt32(1), dr.GetInt32(2)};

							_allPagesInfo[dr.GetString(0)] = aPageInfo;
						}
						dr.Close();

						//   ɽ÷ 
						Context.Cache.Insert("_AllPagesInfo", _allPagesInfo, null, DateTime.Now.AddMinutes(30), TimeSpan.Zero);
					}
				}
				return _allPagesInfo;
			}
		}

		//ڷ ־  URL Ѵ.
		public string GetLink(string pageName)
		{
			//string link = Request.Path + "?" + GetPageParams(pageName);
			string link = Request.ApplicationPath + "DesktopDefault.aspx?" + GetPageParams(pageName);

			return link;
		}

		//ڷ ־  Ÿ  ڿ ȯѴ.
		public string GetPageParams(string pageName)
		{
			object oTmp = AllPagesInfo[pageName];
			if (oTmp == null)
				throw new ArgumentException(pageName + "  ϴ.");

			int[] aInfo = (int[])oTmp;

			return "psup=tabid=" + aInfo[0] + ":pageid=" + aInfo[1];
		}

		// ⿡ ϴ Ű Ʈ Ѵ
		public void SetObjectToUniqueCache(string key, object toCache, int lifeMinute)
		{
			Context.Cache.Insert("__Obj_" + CacheKey + key, toCache, null, DateTime.Now.AddMinutes(lifeMinute), TimeSpan.Zero, CacheItemPriority.High, null);
		}

		// ⿡ ϴ Ű Ʈ ´
		public object GetObjectFromUniqueCache(string key)
		{
			return Context.Cache["__Obj_" + CacheKey + key];
		}

		// ⿡ ϴ Ű Ʈ ɽÿ Ѵ.
		public void RemoveObjectUniqueCache(string key)
		{
			Context.Cache.Remove("__Obj_" + CacheKey + key);
		}

		//ٸ    Ű Ʈ Ѵ.
		public void SetObjectToCache(string key, object toCache, int lifeMinute)
		{
			Context.Cache.Insert("__Obj_"+key, toCache, null, DateTime.Now.AddMinutes(lifeMinute), TimeSpan.Zero, CacheItemPriority.High, null);
		}

		//ٸ    Ű Ʈ ´.
		public object GetObjectFromCache(string key)
		{
			return Context.Cache["__Obj_"+key];
		}

		//ٸ    Ű Ʈ ɽÿ Ѵ.
		public void RemoveObjectCache(string key)
		{
			Context.Cache.Remove("__Obj_"+key);
		}
	}


    //*********************************************************************
    //
    // CachedPortalModuleControl Class
    //
    // The CachedPortalModuleControl class is a custom server control that
    // the Portal framework uses to optionally enable output caching of 
    // individual portal module's content.
    //
    // If a CacheTime value greater than 0 seconds is specified within the 
    // Portal.Config configuration file, then the CachePortalModuleControl
    // will automatically capture the output of the Portal Module User Control
    // it wraps.  It will then store this captured output within the ASP.NET
    // Cache API.  On subsequent requests (either by the same browser -- or
    // by other browsers visiting the same portal page), the CachedPortalModuleControl
    // will attempt to resolve the cached output out of the cache.
    //
    // Note: In the event that previously cached output can't be found in the
    // ASP.NET Cache, the CachedPortalModuleControl will automatically instatiate
    // the appropriate portal module user control and place it within the
    // portal page.
    //
    //*********************************************************************

    public class CachedPortalModuleControl : Control {

        // Private field variables

        private ModuleSettings  _moduleConfiguration;
        private String          _cachedOutput = "";
        private int             _portalId = 0;

        //    ϴ ModuleSettings ü
        public ModuleSettings ModuleConfiguration {
            get {
                return _moduleConfiguration;
            }
            set {
                _moduleConfiguration = value;
            }
        }

		//  ID
        public int ModuleId {
            get {
                return _moduleConfiguration.ModuleId;
            }
        }

		// Ż ID
        public int PortalId {
            get {
                return _portalId;
            }
            set {
                _portalId = value;
            }
        }

        //*********************************************************************
        //
        // CacheKey Property
        //
        // The CacheKey property is used to calculate a "unique" cache key
        // entry to be used to store/retrieve the portal module's content
        // from the ASP.NET Cache.
        //
        //*********************************************************************

        public String CacheKey {
            get {
                return "Key:" + this.GetType().ToString() + this.ModuleId + PortalSecurity.IsInRoles(_moduleConfiguration.AuthorizedEditRoles);
            }
        }

        //*********************************************************************
        //
        // CreateChildControls Method
        //
        // The CreateChildControls method is called when the ASP.NET Page Framework
        // determines that it is time to instantiate a server control.
        // 
        // The CachedPortalModuleControl control overrides this method and attempts
        // to resolve any previously cached output of the portal module from the 
        // ASP.NET cache.  
        //
        // If it doesn't find cached output from a previous request, then the
        // CachedPortalModuleControl will instantiate and add the portal module's
        // User Control instance into the page tree.
        //
        //*********************************************************************

		protected override void CreateChildControls() 
		{
			// ASP.NET ĳÿ  ĳõ ͸ ´.
			if (_moduleConfiguration.CacheTime > 0) {
				_cachedOutput = (String) Context.Cache[CacheKey];
			}

			// û ĳõ Ͱ ٸ
			if (_cachedOutput == null) {
				base.CreateChildControls();

				//PortalModuleControl Ʈ Ѵ.
				PortalModuleControl module = (PortalModuleControl) Page.LoadControl(_moduleConfiguration.DesktopSrc);
                
				module.ModuleConfiguration = this.ModuleConfiguration;
				module.PortalId = this.PortalId;
				//PortalModuleControl Ʈ ڽμ ߰Ѵ.
				this.Controls.Add(module);
			}
		}

        //*********************************************************************
        //
        // Render Method
        //
        // The Render method is called when the ASP.NET Page Framework
        // determines that it is time to render content into the page output stream.
        // 
        // The CachedPortalModuleControl control overrides this method and captures
        // the output generated by the portal module user control.  It then 
        // adds this content into the ASP.NET Cache for future requests.
        //
        //*********************************************************************

		protected override void Render(HtmlTextWriter output) 
		{
			//ĳ Ÿ 0̸  ĳ ʴ´.
			if (_moduleConfiguration.CacheTime == 0) {
				base.Render(output);
				return;
			}

			// If no cached output was found from a previous request, render
			// child controls into a TextWriter, and then cache the results
			// in the ASP.NET Cache for future requests.

			if (_cachedOutput == null) {
				//ڽ Ʈ(PortalModuleControl)   
				// TextWriter ü Ѵ.
				TextWriter innerWriter = new StringWriter();

				//  TextWriter ü ڷ  
				//HtmlTextWriter ü Ѵ
				HtmlTextWriter tempWriter = new HtmlTextWriter(innerWriter);
				base.Render(tempWriter);
				//ڽ Ʈѿ(PortalModuleControl)  
				// ´.
				_cachedOutput = tempWriter.InnerWriter.ToString();

				//  3  ̷ ص .
				//output.InnerWriter = innerWriter;
				//base.Render(output);
				//_cachedOutput = output.InnerWriter.ToString();

				Context.Cache.Insert(CacheKey, _cachedOutput, null, DateTime.Now.AddSeconds(_moduleConfiguration.CacheTime), TimeSpan.Zero);
			}

			// ĳõ ͸  Ѵ.
			output.Write(_cachedOutput);
		}
    }


	/*************************************
	 *  Ŭ  ̿ϼ
	 ************************************/
	/*
	public enum CacheModeType {TimeOnly, DatabaseLocally, DatabaseRemotely};

	public class CachedPortalModuleControlEx : Control
	{
		
		// Private field variables

		private ModuleSettings  _moduleConfiguration;
		private String          _cachedOutput = "";
		private int             _portalId = 0;
		//private int				_cacheTime = 0;
		//private CacheModeType		_cacheMode = CacheModeType.TimeOnly;
		//private string			_dependencyString;
		//private DateTime		_ValidTime = DateTime.Now;

		// Public property accessors

		public ModuleSettings ModuleConfiguration {

			get {
				return _moduleConfiguration;
			}
			set {
				_moduleConfiguration = value;
			}
		}

		public int ModuleId {

			get {
				return _moduleConfiguration.ModuleId;
			}
		}

		public int PortalId {

			get {
				return _portalId;
			}
			set {
				_portalId = value;
			}
		}

		//public void SetCache(int expTime, CacheModeType cacheMode, string dependencyString)
		//{
		//	//_cacheTime = expTime;
		//	_cacheMode = cacheMode;
		//	if (cacheMode != CacheModeType.TimeOnly && (dependencyString == null || dependencyString == ""))
		//		throw(new ArgumentException());

		//	_dependencyString = dependencyString;
		//}

		//*********************************************************************
		//
		// CacheKey Property
		//
		// The CacheKey property is used to calculate a "unique" cache key
		// entry to be used to store/retrieve the portal module's content
		// from the ASP.NET Cache.
		//
		//*********************************************************************

		public String CacheKey {
			get {
				return "Key:" + this.GetType().ToString() + this.ModuleId + PortalSecurity.IsInRoles(_moduleConfiguration.AuthorizedEditRoles);
			}
		}

		protected override void CreateChildControls() 
		{
			DateTime _ValidTime = new DateTime(1, 1, 1);

			// Attempt to resolve previously cached content from the ASP.NET Cache

			if (_moduleConfiguration.CacheTime > 0) 
			{
				//ɽð ϵ ð  ɽÿ  ɽô ׻ 
				//CacheKey+"Time"  ϴ ð CacheModeType.DatabaseRemotely  츸 ʿϴ.
				//⼭  ð ͺ̽   ð Ѵ.
				object oCachedTime = Context.Cache[CacheKey+"Time"];
				if (oCachedTime != null)
					_ValidTime = (DateTime) Context.Cache[CacheKey+"Time"];
				else
					Context.Cache.Remove(CacheKey);

				_cachedOutput = (String) Context.Cache[CacheKey];

				//Ÿ̽ Ȱ ִ Ѵ.
				if (_cachedOutput != null && _moduleConfiguration.CacheMode == CacheModeType.DatabaseRemotely)
				{
					SqlConnection myConnection = new SqlConnection(ConfigurationSettings.AppSettings["connectionString"]);
					SqlCommand myCommand = new SqlCommand(_moduleConfiguration.CacheDependencyString, myConnection);

					myConnection.Open();
					SqlDataReader dr = myCommand.ExecuteReader();

					dr.Read();
					if (_ValidTime < dr.GetDateTime(0))
					{
						_cachedOutput = null;
						Context.Cache[CacheKey+"Time"] = dr.GetDateTime(0);
					}

					myConnection.Close();
				}
			}

			// If no cached content is found, then instantiate and add the portal
			// module user control into the portal's page server control tree

			if (_cachedOutput == null) 
			{

				base.CreateChildControls();

				PortalModuleControlEx module = (PortalModuleControlEx) Page.LoadControl(_moduleConfiguration.DesktopSrc);
                
				module.ModuleConfiguration = this.ModuleConfiguration;
				module.PortalId = this.PortalId;
				module.ParentCacheKey = this.CacheKey;

				this.Controls.Add(module);
			}
		}

		protected override void Render(HtmlTextWriter output) 
		{

			// If no caching is specified, render the child tree and return 

			if (_moduleConfiguration.CacheTime == 0)
			{
				base.Render(output);
				return;
			}

			if (_cachedOutput == null) 
			{
				TextWriter tempWriter = new StringWriter();
				base.Render(new HtmlTextWriter(tempWriter));
				_cachedOutput = tempWriter.ToString();

				CacheDependency oDepen;
				if (_moduleConfiguration.CacheMode == CacheModeType.DatabaseLocally)
					oDepen = new CacheDependency(_moduleConfiguration.CacheDependencyString);
				else
					oDepen = null;
				Context.Cache.Insert(CacheKey, _cachedOutput, oDepen, DateTime.Now.AddMinutes(_moduleConfiguration.CacheTime), TimeSpan.Zero);
				Context.Cache.Insert(CacheKey+"Time", DateTime.Now, null, DateTime.Now.AddMinutes(_moduleConfiguration.CacheTime), TimeSpan.Zero);
			}

			// Output the user control's content

			output.Write(_cachedOutput);
		}
	}
	*/
}
